<?php

namespace App\Service;


use Illuminate\Support\Facades\Log;

class JRFIDServerService extends Service
{
    public function loginRule($data){
        if(empty($data['name'])) return [false, '请输入账号！'];
        if(empty($data['password'])) return [false, '请输入密码！'];

        list($status, $msg) = $this->getToken($data);
        if(! $status) return [false, $msg];

        return [true, ['data' => $msg]];
    }

    public function getToken($data){
        $account = $data['name'];
        $password = $data['password'];

        $url = config("j_rfid.login");
        $post = [
            "name" => $account,
            "password" => $password,
            "rememberMe" => true
        ];
        $header = ['Content-Type:application/json'];
        list($status, $result) = $this->post_helper($url,json_encode($post), $header);
        if(! $status) return [false, $result];

        //登录失败
        if(! empty($result['errorMessage'])) return [false, $result['errorMessage']];

        return [true, $result];
    }

    public function getSite($data){
        $url = config("j_rfid.site");
        list($status,$result) = $this->get_helper($url);
        if(! $status) return [false, $result];

        if(! empty($result['errorMessage'])) return [false, $result['errorMessage']];

        return [true, $result];
    }

    public function getSite2($data,$param){
        $url = config("j_rfid.site2");

        $header = ["Authorization: {$param['token']}"];
        list($status,$result) = $this->get_helper($url,$header);
        if(! $status) return [false, $result];

        if(! empty($result['errorMessage'])) return [false, $result['errorMessage']];

        return [true, $result];
    }

    public function getFlowByProduce($data,$param){
        if(empty($data['produce_no'])) return [false, '订单号不能为空'];
        if(empty($data['site'])) return [false, '站点不能为空'];

        $url = config("j_rfid.get_flow_by_produce");
        $post = [
            'produce_no' => $data['produce_no'],
            'site' => $data['site'],
        ];
        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        return [true, $result['data']];
    }

    public function getProduceByContract($data,$param){
        if(empty($data['contract_no'])) return [false, '合同不能为空'];
        if(empty($data['site'])) return [false, '站点不能为空'];

        $url = config("j_rfid.get_produce_by_contract");
        $post = [
            'contract_no' => $data['contract_no'],
            'site' => $data['site'],
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        return [true, $result['data']];
    }

    public function getPrintData($data){
        if(empty($data['id'])) return [false, '数据ID不能为空'];
        if(empty($data['type'])) return [false, '打印数据类型不能为空'];

        $size = $data['size'] ?? 10;
        $number = $data['number'] ?? 1;

        $rsaService = new RsaEncryptionService();
        $dataToEncrypt = [
            'id' => $data['id'],
            'type' => $data['type'],
            'size' => $size,
            'number' => $number,
        ];
        $encryptedData = $rsaService->encrypt(json_encode($dataToEncrypt));

        $url = config("j_rfid.get_print_data");
        $post = [
            'body' => $encryptedData,
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),['Content-Type:application/json']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['message']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['furn_pro_flow_dt_a'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function getTeam($data,$param){
        if(empty($data['site'])) return [false, '站点不能为空'];
        $header = array_merge($param['header'], ['site:'. $data['site']]);
        $url = config("j_rfid.get_team");
        $post['rules'] = [
            [
                'field' => 'site',
                'option' => 'LIKE_ANYWHERE',
                'values' => [$data['site']]
            ],
            [
                'field' => 'is_used',
                'option' => 'IN',
                'values' => ['1']
            ]
        ];
        $post['size'] = $data['size'] ?? 6;
        $post['number'] = ($data['number'] ?? 1) - 1;

        list($status,$result) = $this->post_helper($url,json_encode($post),$header);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];
        if(! isset($result['content'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function completionOrders($data,$param){
        //{"type":"","id":[],"completed_teams_group":"","completed_teams_group_show":"","completed_date":"","completed_date_lt":""}
//        $p = [
//            'screenDataList' =>[
//                [
//                    'id' => [], //产品
//                    'type' => ''
//                ],
//                [
//                    'id' => [],//组件
//                    'type' => ''
//                ],
//                [
//                    'id' => [],//部件
//                    'type' => ''
//                ],
//            ],
//            'completed_teams_group' => "",
//            'completed_teams_group_show' => "",
//            'procedure_code' => [],
//            'site' => ""
//        ];dd(json_encode($p));

        if(empty($data['screenDataList'])) return [false, '完工数据不能为空'];
        if(empty($data['site'])) return [false, '站点信息不能为空'];

        list($date,$utc_date) = $this->getE8AndUTC();
        foreach ($data['screenDataList'] as $key => $value){
            if(empty($value['id']) || ! isset($value['type'])) return [false, '完工数据不能为空'];
            $data['screenDataList'][$key]['completed_teams_group'] = $data['completed_teams_group'] ?? "";
            $data['screenDataList'][$key]['completed_teams_group_show'] = $data['completed_teams_group_show'] ?? "";
            $data['screenDataList'][$key]['completed_date'] = $utc_date;
            $data['screenDataList'][$key]['completed_date_lt'] = $date;
            $data['screenDataList'][$key]['procedure_code'] = $data['procedure_code'] ?? [];
            $data['screenDataList'][$key]['site'] = $data['site'] ?? "";
        }

        $url = config("j_rfid.completion_orders");
        $post = [
            'screenDataList' => $data['screenDataList'],
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post), $param['header']);
        if(! $status) return [false, $result];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ''];
    }

    public function qualityOrders($data,$param){
        if(empty($data['screenDataList'])) return [false, '质检数据不能为空'];
        if(empty($data['site'])) return [false, '站点信息不能为空'];

        list($date,$utc_date) = $this->getE8AndUTC();
        foreach ($data['screenDataList'] as $key => $value){
            if(empty($value['id']) || ! isset($value['type'])) return [false, '质检数据不能为空'];
            $data['screenDataList'][$key]['qualified_teams_group'] = $data['qualified_teams_group'] ?? "";
            $data['screenDataList'][$key]['qualified_teams_group_show'] = $data['qualified_teams_group_show'] ?? "";
            $data['screenDataList'][$key]['qualified_date'] = $utc_date;
            $data['screenDataList'][$key]['qualified_date_lt'] = $date;
            $data['screenDataList'][$key]['procedure_code'] = $data['procedure_code'] ?? [];
            $data['screenDataList'][$key]['site'] = $data['site'] ?? "";
        }

        $url = config("j_rfid.quality_orders");
        $post = [
            'screenDataList' => $data['screenDataList'],
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post), $param['header']);
        if(! $status) return [false, $result];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ''];
    }

    public function screenPrint($data){
        if(empty($data['id'])) return [false, '数据ID不能为空'];
        if(empty($data['type'])) return [false, 'TYPE不能为空'];

        $rsaService = new RsaEncryptionService();
        $dataToEncrypt = [
            'id' => $data['id'],
            'type' => $data['type']
        ];
        $encryptedData = $rsaService->encrypt(json_encode($dataToEncrypt));

        $url = config("j_rfid.screen_print");
        $post = [
            'body' => $encryptedData,
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),['Content-Type:application/json']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['message']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        return [true, ''];
    }

    public function getE8AndUTC(){
        $eastEightTimeZone = new \DateTimeZone('Asia/Shanghai'); // 东八区时区
        $utcTimeZone = new \DateTimeZone('UTC'); // UTC时区

        // 假设的东八区时间，没有毫秒部分
        $dateTimeString = date("Y-m-d H:i:s");
        $milliseconds = '000'; // 假设的毫秒值

        // 创建DateTime对象并设置时区为东八区
        $dateTime = new \DateTime($dateTimeString, $eastEightTimeZone);

        // 将DateTime对象转换为UTC时区
        $dateTime->setTimezone($utcTimeZone);

        // 格式化DateTime对象为ISO 8601格式，并添加毫秒部分
        $formattedDate = $dateTime->format('Y-m-d\TH:i:s') . '.' . str_pad($milliseconds, 3, '0', STR_PAD_LEFT) . 'Z';

        return [$dateTimeString,$formattedDate];
    }

    public function getContractDrawRoom($data,$param){
        if(empty($data['site'])) return [false, '站点不能为空'];
        if(empty($data['type'])) return [false, 'TYPE不能为空'];
        $post['site'] = $data['site'];
        $post['type'] = $data['type'];

        $url = config("j_rfid.get_contract_draw_room");
        if(isset($data['contract_no'])) $post['contract_no'] = $data['contract_no'];
        if(isset($data['room_no'])) $post['room_no'] = $data['room_no'];
        if(isset($data['drawing_no'])) $post['drawing_no'] = $data['drawing_no'];
        $post['size'] = $data['size'] ?? 10;
        $post['number'] = $data['number'] ?? 1;

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result['data']];
    }

    public function screenGetProFlow($data,$param){
        if(empty($data['site'])) return [false, '站点不能为空'];
        if(empty($data['contract_no'])) return [false, '合同不能为空'];
        if(empty($data['drawing_no'])) return [false, '图号不能为空'];
        if(empty($data['room_no'])) return [false, '房间号不能为空'];

        $url = config("j_rfid.screen_get_pro_flow");

        $post = [
            'site' => $data['site'],
            'contract_no' => $data['contract_no'],
            'drawing_no' => $data['drawing_no'],
            'room_no' => $data['room_no'],
        ];
        $post['size'] = $data['size'] ?? 10;
        $post['number'] = $data['number'] ?? 1;

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result['data']];
    }

    public function screenSend($data,$param){
        if(empty($data['screenDataList'])) return [false, '收发货数据不能为空'];
        if(empty($data['site']) || empty($data['site_show'])) return [false, '站点信息不能为空'];
        list($date,$utc_date) = $this->getE8AndUTC();
        foreach ($data['screenDataList'] as $key => $value){
            if(empty($value['id']) || ! isset($value['type'])) return [false, '收发货数据不能为空'];
            $data['screenDataList'][$key]['delivery_teams_group'] = $data['delivery_teams_group'];
            $data['screenDataList'][$key]['delivery_teams_group_show'] = $data['delivery_teams_group_show'];
            $data['screenDataList'][$key]['delivery_date'] = $utc_date;
            $data['screenDataList'][$key]['delivery_date_lt'] = $date;
            $data['screenDataList'][$key]['site'] = $data['site'];
            $data['screenDataList'][$key]['site_show'] = $data['site_show'];
        }

        $url = config("j_rfid.screen_send");
        $post = [
            'screenDataList' => $data['screenDataList'],
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ''];
    }

    public function screenPackage($data,$param){
        if(empty($data['screenDataList'])) return [false, '包装数据不能为空'];
        if(empty($data['site']) || empty($data['site_show'])) return [false, '站点信息不能为空'];
        list($date,$utc_date) = $this->getE8AndUTC();
        foreach ($data['screenDataList'] as $key => $value){
            if(empty($value['id']) || ! isset($value['type'])) return [false, '包装数据不能为空'];
            $data['screenDataList'][$key]['package_teams_group'] = $data['package_teams_group'];
            $data['screenDataList'][$key]['package_teams_group_show'] = $data['package_teams_group_show'];
            $data['screenDataList'][$key]['package_date'] = $utc_date;
            $data['screenDataList'][$key]['package_date_lt'] = $date;
            $data['screenDataList'][$key]['site'] = $data['site'];
            $data['screenDataList'][$key]['site_show'] = $data['site_show'];
        }

        $url = config("j_rfid.screen_package");
        $post = [
            'screenDataList' => $data['screenDataList'],
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, ['box_no' => $result['data'] ?? [], 'package_date' => $date]];
    }

    public function screenGetPackageData($data,$param){
        if(empty($data['site'])) return [false, '站点信息不能为空'];
        if(empty($data['contract_no'])) return [false, '合同不能为空'];
//        if(empty($data['drawing_no'])) return [false, '图号不能为空'];
//        if(empty($data['room_no'])) return [false, '房间号不能为空'];

        $size = $data['size'] ?? 10;
        $number = $data['number'] ?? 1;

        $url = config("j_rfid.screen_get_package_data");
        $post = [
            'site' => $data['site'],
            'contract_no' => $data['contract_no'],
            'drawing_no' => $data['drawing_no'] ?? "",
            'room_no' => $data['room_no'] ?? "",
            'size' => $size,
            'number' => $number,
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['msg']];

        if(! isset($result['success'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result['data']];
    }

    public function screenGetProDtData($data,$param){
        if(empty($data['id'])) return [false, '数据ID不能为空'];
        if(empty($data['type'])) return [false, 'TYPE不能为空'];
//        if(empty($data['contract_no'])) return [false, '合同不能为空'];
//        if(empty($data['drawing_no'])) return [false, '图号不能为空'];

        $url = config("j_rfid.screenGetProDtData");
        $post = [
            'id' => $data['id'],
            'type' => $data['type'],
            'contract_no' => $data['contract_no'] ?? "",
            'drawing_no' => $data['drawing_no'] ?? "",
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['message']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['furn_pro_flow_dt_a'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        return [true, $result];
    }

    public function screenGetPackageAssem($data,$param){
        if(empty($data['id'])) return [false, '数据ID不能为空'];
        if(empty($data['type'])) return [false, 'TYPE不能为空'];
        if(empty($data['site'])) return [false, '站点不能为空'];

        $url = config("j_rfid.screenGetPackageAssem");
        $post = [
            'id' => $data['id'],
            'type' => $data['type'],
            'site' => $data['site'],
            'contract_no' => $data['contract_no'] ?? ""
        ];

        list($status,$result) = $this->post_helper($url,json_encode($post),$param['header']);
        if(! $status) return [false, $result];

        if(! empty($result['status']) && $result['status'] == 'error') return [false, $result['message']];
        if(! empty($result['type']) && $result['type'] == 'errorVm') return [false, $result['message']];

        if(! isset($result['assembly_list'])) {
            $error = $result[0]['message'] ?? "操作失败，请刷新页面";
            return [false, $error];
        }

        foreach ($result['assembly_list'] as $key => $value){
            foreach ($value['package_assembly_list'] as $kk => $vv){
                $result['assembly_list'][$key]['package_assembly_list'][$kk]['package_date'] = $this->changeDateToDateMin($vv['package_date']);
            }
        }

        return [true, $result['assembly_list']];
    }

    public function post_helper($url, $data, $header = [], $timeout = 20){
        Log::channel('apiLog')->info('工装云POST', ["api" => $url , "param" => $data ,"header" => $header]);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch,  CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_ENCODING, '');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);

        if(!is_null($data)) curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        $r = curl_exec($ch);

        if ($r === false) {
            // 获取错误号
            $errorNumber = curl_errno($ch);
            // 获取错误信息
            $errorMessage = curl_error($ch);
            $message = "cURL Error #{$errorNumber}: {$errorMessage}";

            Log::channel('apiLog')->info('工装云POST结果', ["message" => $message]);
            return [false, $message];
        }
        curl_close($ch);

        Log::channel('apiLog')->info('工装云POST结果', ["message" => json_decode($r, true)]);

        return [true, json_decode($r, true)];
    }

    public function get_helper($url,$header=[],$timeout = 20){
        $ch = curl_init();
        curl_setopt_array($ch, array(
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => $timeout,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_HTTPHEADER => $header,
        ));
        $r = curl_exec($ch);

        if ($r === false) {
            // 获取错误号
            $errorNumber = curl_errno($ch);
            // 获取错误信息
            $errorMessage = curl_error($ch);
            $message = "cURL Error #{$errorNumber}: {$errorMessage}";
            Log::channel('apiLog')->info('工装云GET', ["message" => $message]);
            return [false, $message];
        }

        curl_close($ch);
        Log::channel('apiLog')->info('工装云GET', ["message" => json_decode($r, true)]);

        return [true, json_decode($r, true)];
    }
}
